"""
The MIT License (MIT)

Copyright (c) 2021-present UnrealFar & TheGenocides
Copyright (c) 2023-present Sengolda

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from __future__ import annotations

import logging
import datetime
from typing import Any, List, Dict, TYPE_CHECKING
from .utils import time_parse_todt
from .objects import Comparable

if TYPE_CHECKING:
    from .client import Client
    from .type import ID

_log = logging.getLogger(__name__)


class Environment(Comparable):
    """Represents a dev environment to use one of the subscription APIs (Account Activity API or events etc)

    .. versionadded:: 1.5.0
    """

    __slots__ = ("_payload", "client")

    def __init__(self, data: Dict[str, Any], *, client: Client):
        self._payload = data
        self.client = client
        super().__init__(self.name)

    def __repr__(self) -> str:
        return f"Environment(name={self.name})"

    @property
    def name(self) -> str:
        """:class:`str`: The environment name/label.

        .. versionadded:: 1.5.0
        """
        return self._payload.get("environment_name")

    @property
    def label(self) -> str:
        """:class:`str`: An alias to :meth:`Environment.name`

        .. versionadded:: 1.5.0
        """
        return self.name

    @property
    def webhooks(self) -> List[Webhook]:
        """List[:class:`Webhook`]: Returns a list of webhooks.

        .. versionadded:: 1.5.0
        """
        return [Webhook(data, environment=self, client=self.client) for data in self._payload.get("webhooks")]

    def add_user_subscription(self, client: Client) -> None:
        """Add a new user subscription to the environment, which is the client (that you passed in the argument) itself.

        .. note::
            If you want to add other user subscription, use :meth:`OauthSession.create_oauth_url` to generate an oauth url and get the oauth token and verifier, then use :meth:`OauthSession.post_oauth_token` to post the oauth token.


        .. versionadded:: 1.5.0
        """
        client.http.request(
            "POST",
            "1.1",
            f"/account_activity/all/{self.label}/subscriptions.json",
            auth=True,
        )

    def add_my_subscription(self) -> None:
        """Add a new user subscription to the environment, which is the client WHO made the environment request. Use :meth:`add_user_subscription` to add other user subscription. This method only add the client WHO made the fetch environment request.


        .. versionadded:: 1.5.0
        """
        self.client.http.request(
            "POST",
            "1.1",
            f"/account_activity/all/{self.label}/subscriptions.json",
            auth=True,
        )

    def register_webhook(self, url: str) -> Webhook:
        """Register your WebHook with your WebApp's url that you develop. Before this, you need to develop, deploy and host a WebApp that will receive Twitter webhook events. You also need to perform a Twitter Challenge Response Check (CRC) GET request and responds with a properly formatted JSON response.

        Parameters
        ------------
        url: :class:`str`
            Your WebApp url that you want to register as the WebHook url. Twitter will send account events to this url as an http post request.'

        Returns
        ---------
        :class:`Webhook`
            This method returns a :class:`Webhook` object.


        .. versionadded:: 1.5.0
        """
        res = self.client.http.request(
            "POST",
            "1.1",
            f"/account_activity/all/{self.label}/webhooks.json",
            auth=True,
            params={"url": url},
        )
        return Webhook(res, environment=self, client=self)

    def fetch_all_subscriptions(self) -> List[int]:
        """Returns a list of the the current users subscriptions from the environment.

        Returns
        ---------
        List[:class:`int`]
            This method returns a list of :class:`int` object.


        .. versionadded:: 1.5.0
        """
        res = self.client.http.request("GET", "1.1", f"/account_activity/all/{self.label}/subscriptions/list.json")

        return [int(subscription.get("user_id")) for subscription in res.get("subscriptions")]


class Webhook(Comparable):
    """Represents a webhook for an environment. This webhook belongs to an environment and have a webhook url for sending account activity events.

    .. versionadded:: 1.5.0
    """

    __slots__ = ("_id", "_url", "_valid", "_environment", "client")

    def __init__(self, data: Dict[str, Any], *, environment: Environment, client: Client):
        self._id = data.get("id")
        self._url = data.get("url")
        self._valid = data.get("valid")
        self._environment = environment
        self.client = client
        super().__init__(self.id)

    def __repr__(self) -> str:
        return f"Webhook(id={self.id} url={self.url} valid={self.valid} environment={self.environment})"

    @property
    def id(self) -> int:
        """:class:`int`: Returns the webhook unique id.

        .. versionadded:: 1.5.0
        """
        return self._id

    @id.setter
    def id(self, obj: ID) -> None:
        self._id = obj

    @property
    def url(self) -> str:
        """:class:`str`: Returns the webhook url.

        .. versionadded:: 1.5.0
        """
        return self._url

    @url.setter
    def url(self, obj: str) -> None:
        self._url = obj

    @property
    def valid(self) -> bool:
        """:class:`bool`: Returns True if the webhook is valid else False.

        .. versionadded:: 1.5.0
        """
        return self._valid

    @valid.setter
    def valid(self, obj: bool) -> None:
        if not isinstance(obj, bool):
            raise ValueError("Webhook.valid setter method can only set bool object!")
        self._valid = obj

    @property
    def environment(self) -> Environment:
        """:class:`Environment`: Returns an environment where the webhook belongs to.

        .. versionadded:: 1.5.0
        """
        return self._environment

    @property
    def env(self):
        """:class:`Environment`: An alias to :meth:`Webhook.environment`

        .. versionadded:: 1.5.0
        """
        return self.environment

    @property
    def created_at(self) -> datetime.datetime:
        """:class:`datetime.datetime`: Returns a datetime.datetime object with the webhook's created timestamp.

        .. versionadded:: 1.5.0
        """
        return time_parse_todt(self._payload.get("created_at"))

    def delete(self) -> Webhook:
        """Delete the webhook.

        Returns
        ---------
        :class:`Webhook`
            Returns the webhook object with the valid set to False.


        .. versionadded:: 1.5.0
        """
        self.client.http.request(
            "DELETE",
            "1.1",
            f"/account_activity/all/{self.env.label}/webhooks/{self.id}.json",
            auth=True,
        )
        self.valid = False
        return self

    def trigger_crc(self) -> bool:
        """Trigger a challenge-response-checks to enable Twitter to confirm the ownership of the WebApp receiving webhook events. Will return True if its successful else False.

        Returns
        ---------
        :class:`bool`
            This method returns a :class:`bool` object.


        .. versionadded:: 1.5.0
        """
        _log.info("Triggering a CRC Challenge.")

        if not self.client.environment or not self.client.webhook:
            _log.warn("CRC Failed: client is not listening! use the listen method at the very end of your file!")
            return False

        self.client.http.request(
            "PUT",
            "1.1",
            f"/account_activity/all/{self.env.label}/webhooks/{self.id}.json",
            auth=True,
        )
        _log.info("Successfully triggered a CRC.")
        return True



"""
The MIT License (MIT)

Copyright (c) 2021-present UnrealFar & TheGenocides
Copyright (c) 2023-present Sengolda

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from __future__ import annotations

import datetime
import requests
import json

from typing import Optional, List, TYPE_CHECKING
from .objects import Comparable
from .utils import time_parse_todt
from .enums import JobType, JobStatus, JobResultAction, JobResultActionReason
from .dataclass import JobResult

if TYPE_CHECKING:
    from .http import HTTPClient
    from .type import ID, Payload


class Job(Comparable):
    """Represents a job compliance.

    .. versionadded:: 1.5.0
    """

    __slots__ = ("__payload", "http_client")

    def __init__(self, data: Payload, *, http_client: Optional[HTTPClient] = None):
        self.__payload = data
        self.http_client = http_client
        super().__init__(self.id)

    def __repr__(self):
        return f"Job(name={self.name} id={self.id} type={self.type})"

    @property
    def name(self) -> str:
        """:class:`str`: Returns the job's name

        .. versionadded:: 1.5.0
        """
        return self.__payload.get("name")

    @property
    def id(self) -> ID:
        """:class:`id`: Returns the job's id

        .. versionadded:: 1.5.0
        """
        return int(self.__payload.get("id"))

    @property
    def resumable(self) -> bool:
        """:class:`bool`: Returns True if the job is esumable otherwise False.

        .. versionadded:: 1.5.0
        """
        return self.__payload.get("resumable")

    @property
    def download_url(self) -> str:
        """:class:`str`: Returns the job's download url.

        .. versionadded:: 1.5.0
        """
        return self.__payload.get("download_url")

    @property
    def upload_url(self) -> str:
        """:class:`str`: Returns the job's upload url.

        .. versionadded:: 1.5.0
        """
        return self.__payload.get("upload_url")

    @property
    def type(self) -> JobType:
        """:class:`JobType`: Returns :class:`JobType` with the job's type.

        .. versionadded:: 1.5.0
        """
        return JobType(self.__payload.get("type"))

    @property
    def status(self) -> JobStatus:
        """:class:`JobStatus`: Returns :class:`JobStatus` with the job's status.

        .. versionadded:: 1.5.0
        """
        return JobStatus(self.__payload.get("status"))

    @property
    def created_at(self) -> datetime.datetime:
        """:class:`datetime.datetime`: Returns a datetime.datetime object with the job's created timestamp.

        .. versionadded:: 1.5.0
        """
        return time_parse_todt(self.__payload.get("created_at"))

    @property
    def download_expires_at(self) -> datetime.datetime:
        """:class:`datetime.datetime`: Returns a datetime.datetime object which the download URL will be available (usually seven days from the request time).

        .. versionadded:: 1.5.0
        """
        return time_parse_todt(self.__payload.get("download_expires_at"))

    @property
    def upload_expires_at(self) -> datetime.datetime:
        """:class:`datetime.datetime`: Returns a datetime.datetime object which the upload URL will be available (usually seven days from the request time).

        .. versionadded:: 1.5.0
        """
        return time_parse_todt(self.__payload.get("upload_expires_at"))

    def upload_file(self, path_to_filename: str):
        """Upload a txt file to :meth:`Job.upload_url`.

        Parameters
        ------------
        path_to_filename: :class:`str`
            Path to the filename that you want to upload.


        .. versionadded:: 1.5.0
        """
        requests.put(self.upload_url, data=open(path_to_filename, "rb"), headers={"Content-Type": "text/plain"})

    def get_download_result(self) -> Optional[List[JobResult]]:
        """Get the download result of the job. You can only get the result if the job has a `complete` status, check :meth:`Job.status` to check the job's status.

        Returns
        ---------
        Optional[List[:class:`JobResult`]]
            This method returns a list of :class:`JobResult` objects.


        .. versionadded:: 1.5.0
        """
        res = requests.get(self.download_url)
        results = []
        for data in res.text.splitlines():
            if not data:
                return None

            if isinstance(data, str):
                data = json.loads(data)

            data["action"] = JobResultAction(data["action"])
            data["reason"] = JobResultActionReason(data["reason"])
            results.append(JobResult(**data))

        return results



"""
The MIT License (MIT)

Copyright (c) 2021-present UnrealFar & TheGenocides
Copyright (c) 2023-present Sengolda

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from __future__ import annotations

import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Optional

from .dataclass import ApplicationInfo, InitiatedVia
from .attachments import CTA, File, QuickReply
from .entities import Hashtag, Symbol, Url, UserMention
from .enums import MessageEventTypeEnum, MessageTypeEnum
from .user import User
from .objects import Comparable

if TYPE_CHECKING:
    from .http import HTTPClient
    from .type import ID

__all__ = ("Message", "DirectMessage", "WelcomeMessage", "WelcomeMessageRule")


class Message(Comparable):
    """Represents the base Message of all Message types in Twitter.

    .. versionadded:: 1.2.0
    """

    __slots__ = ("_text", "_id", "_type")

    def __init__(self, text: Optional[str], id: ID, type: int):
        super().__init__(id)
        self._text = text
        self._id = id
        self._type = type

    def __repr__(self) -> str:
        return "Message(text={0.text} id={0.id})".format(self)

    @property
    def text(self) -> str:
        """:class:`str`: Returns the message's text.

        .. versionadded:: 1.2.0
        """
        return self._text

    @property
    def id(self) -> int:
        """:class:`int`: Returns the message's id, or if its a direct message it returns an event id.

        .. versionadded:: 1.2.0
        """
        return int(self._id)

    @property
    def type(self) -> MessageTypeEnum:
        """:class:`MessageTypeEnum`: Returns the message's type.

        .. versionadded:: 1.2.0
        """
        return MessageTypeEnum(self._type)


class DirectMessage(Message):
    """Represents a Direct Message in Twitter. This class inherits :class:`Message`.

    .. versionadded:: 1.2.0
    """

    __slots__ = (
        "__original_payload",
        "_payload",
        "__message_create",
        "__message_data",
        "__entities",
        "_quick_reply_data",
        "_cta_data",
        "http_client",
    )

    def __init__(self, data: Dict[str, Any], *, http_client: HTTPClient):
        self.__original_payload = data
        self._payload = data.get("event", None) or data
        self.__message_create = self._payload.get("message_create", None)
        self.__message_data = self.__message_create.get("message_data", None)
        self.__entities = self.__message_data.get("entities", None)
        self._initiated_via = self._payload.get("initiated_via")
        self._quick_reply_data = self.__message_data.get("quick_reply")
        self._cta_data = self.__message_data.get("ctas")
        self.http_client = http_client
        super().__init__(self.__message_data.get("text"), self._payload.get("id"), 0)

    def __repr__(self) -> str:
        return "DirectMessage(text={0.text} id={0.id} recipient={0.recipient})".format(self)

    @property
    def event_type(self) -> MessageEventTypeEnum:
        """:class:`MessageEventTypeEnum`: Returns the message event type.

        .. versionadded:: 1.2.0
        """
        return MessageEventTypeEnum(self._payload.get("type", None))

    @property
    def recipient(self) -> User:
        """:class:`User`: Returns the user that received the direct message.

        .. versionadded:: 1.2.0
        """
        return self.__message_create.get("target", {}).get("recipient")

    @property
    def sender(self) -> Optional[User]:
        """:class:`User`: Returns the user that sent the direct message.

        .. versionadded:: 1.5.0
        """
        return self.__message_create.get("target", {}).get("sender")

    @property
    def author(self) -> Optional[User]:
        """:class:`User`: An alias to :meth:`DirectMessage.sender`

        .. versionadded:: 1.5.0
        """
        return self.sender

    @property
    def application_info(self) -> Optional[ApplicationInfo]:
        """:class:`ApplicationInfo`: Returns the direct messages's source application info if there is, else it return None.

        .. versionadded:: 1.5.0
        """
        return self.__message_create.get("target", {}).get("source_application", None)

    @property
    def created_at(self) -> datetime.datetime:
        """:class:`datetime.datetime`: Returns the time when the Direct Message event was created.

        .. versionadded:: 1.2.0
        """
        return datetime.datetime.fromtimestamp(int(self._payload.get("created_timestamp")) / 1000)

    @property
    def hashtags(self) -> Optional[List[Hashtag]]:
        """List[:class:`Hashtag`]: Returns the messages's hashtags.

        .. versionadded:: 1.2.0
        """
        return [Hashtag(data) for data in self.__entities.get("hashtags")]

    @property
    def symbols(self) -> Optional[List[Symbol]]:
        """List[:class:`Symbol`]: Returns the messages's symbols.

        .. versionadded:: 1.2.0
        """
        return [Symbol(data) for data in self.__entities.get("symbols")]

    @property
    def mentions(self) -> Optional[List[UserMention]]:
        """List[:class:`UserMention`]: Returns the messages mentioned users.

        .. versionadded:: 1.2.0
        """
        return [UserMention(data) for data in self.__entities.get("user_mentions")]

    @property
    def urls(self) -> Optional[List[Url]]:
        """List[:class:`Url`]: Returns the message's urls.

        .. versionadded:: 1.2.0
        """
        return [Url(data) for data in self.__entities.get("urls")]

    @property
    def quick_reply(self) -> Optional[QuickReply]:
        """Optional[:class:`QuickReply`]: Returns the quick reply attachment in a message, if none found it return None.

        .. versionadded:: 1.3.5
        """
        if self._quick_reply_data and self._quick_reply_data.get("options"):
            attachment = QuickReply(self._quick_reply_data.get("type"))
            for option in self._quick_reply_data.get("options"):
                attachment.add_option(**option)
            return attachment
        return None

    @property
    def quick_reply_response(self) -> Optional[str]:
        """Optional[:class:`str`]: Returns the metadata of the quick reply option that the author clicked.

        .. versionadded:: 1.5.0
        """
        return self.__message_data.get("quick_reply_response", {}).get("metadata", None)

    @property
    def cta(self) -> Optional[CTA]:
        """Optional[:class:`CTA`]: Returns the message's cta attachment.

        .. versionadded:: 1.3.5
        """
        if self._cta_data:
            attachment = CTA()
            for button in self._cta_data:
                attachment.add_button(**button)
            return attachment
        return None

    @property
    def initiated_via(self) -> Optional[InitiatedVia]:
        """Optional[:class:`InitiatedVia`]: Returns :class:`InitiatedVia` object which provides a way to track how a conversation starts. For example, if a welcome message was used then :meth:`InitiatedVia.welcome_message_id` would returns the welcome message id that was used in that conversation.

        .. versionadded:: 1.5.0
        """
        if not self._initiated_via:
            return None

        if not self._initiated_via.get("welcome_message_id"):
            self._initiated_via["welcome_message_id"] = None

        if not self._initiated_via.get("tweet_id"):
            self._initiated_via["tweet_id"] = None

        return InitiatedVia(self._initiated_via)

    def delete(self) -> None:
        """Delete the direct message.

        .. versionadded:: 1.1.0
        """
        self.http_client.request(
            "DELETE",
            "1.1",
            f"/direct_messages/events/destroy.json?id={self.id}",
            auth=True,
        )

        try:
            self.http_client.message_cache.pop(int(self.id))
        except KeyError:
            pass

    def mark_as_read(self):
        """Mark the DirectMessage as read, it also mark other messages before the DirectMessage was sent as read.

        .. versionadded:: 1.3.5
        """
        self.http_client.request(
            "POST",
            "1.1",
            "/direct_messages/mark_read.json",
            params={
                "last_read_event_id": str(self.id),
                "recipient_id": str(self.author.id),
            },
            auth=True,
        )


class WelcomeMessage(Message):
    """Represents a Welcome Message in a Direct Message. This class inherits :class:`Message`.


    .. versionadded:: 1.3.5
    """

    __slots__ = ("_name", "_timestamp", "http_client")

    def __init__(
        self,
        name: Optional[str] = None,
        *,
        text: str,
        id: ID,
        timestamp: str,
        http_client: HTTPClient,
    ):
        super().__init__(text, id, 2)
        self._name = name
        self._timestamp = timestamp
        self.http_client = http_client

    def __repr__(self) -> str:
        return "WelcomeMessage(text={0.text} id={0.id})".format(self)

    @property
    def name(self) -> str:
        """:class:`str`: Returns the welcome message's name.

        .. versionadded:: 1.3.5
        """
        return self._name

    @property
    def created_at(self) -> datetime.datetime:
        """:class:`datetime.datetime`: Returns the welcome message created date.

        .. versionadded:: 1.3.5
        """
        return datetime.datetime.fromtimestamp(int(self._timestamp) / 1000)

    def set_rule(self) -> WelcomeMessageRule:
        """Set a new Welcome Message Rule that determines which Welcome Message will be shown in a given conversation. Returns the created rule if successful.

        .. versionadded:: 1.3.5
        """
        try:
            int(self.id)
        except Exception as e:
            raise e

        data = {"welcome_message_rule": {"welcome_message_id": str(self.id)}}

        res = self.http_client.request(
            "POST",
            "1.1",
            "/direct_messages/welcome_messages/rules/new.json",
            json=data,
            auth=True,
        )

        args = [v for k, v in res.get("welcome_message_rule").items()]
        return WelcomeMessageRule(args[0], args[2], args[1], http_client=self.http_client)

    def update(
        self,
        *,
        text: Optional[str] = None,
        file: Optional[File] = None,
        quick_reply: Optional[QuickReply] = None,
        cta: Optional[CTA] = None,
    ) -> WelcomeMessage:
        """Updates the Welcome Message, you dont need to use set_rule again since this update your default welcome message.

        Parameters
        -----------
        text: Optional[:class:`str`]
            The welcome message main text
        file: Optional[:class:`File`]:
            Represents a single file attachment. It could be an image, gif, or video. It also have to be an instance of pytwot.File
        quick_reply: Optional[:class:`QuickReply`]
            The message's :class:`QuickReply` attachments.
        cta: Optional[:class:`CTA`]
            The message's :class:`CTA` attachment.

        Returns
        ---------
        :class:`WelcomeMessage`
            Returns your :class:`WelcomeMessage` instance.


        .. versionadded:: 1.3.5
        """
        return self.http_client.update_welcome_message(
            welcome_message_id=self.id,
            text=text,
            file=file,
            quick_reply=quick_reply,
            cta=cta,
        )

    def delete(self):
        """Delete the Welcome Message.

        .. versionadded:: 1.3.5
        """
        self.http_client.request(
            "DELETE",
            "1.1",
            "/direct_messages/welcome_messages/destroy.json",
            params={"id": str(self.id)},
            auth=True,
        )


class WelcomeMessageRule(Message):
    """Represents a Welcome Message Rule in a Direct Message. This object is returns by WelcomeMessage.set_rule or client.fetch_welcome_message_rules, it determines which Welcome Message will be shown in a given conversation. This class inherits :class:`Message`.


    .. versionadded:: 1.3.5
    """

    __slots__ = ("_welcome_message_id", "_timestamp", "http_client")

    def __init__(
        self,
        id: ID,
        welcome_message_id: ID,
        timestamp: ID,
        *,
        http_client: HTTPClient,
    ):
        super().__init__(None, id=id, type=3)
        self._welcome_message_id = welcome_message_id
        self._timestamp = timestamp
        self.http_client = http_client

    def __repr__(self) -> str:
        return (
            "WelcomeMessageRule(id={0.id} welcome_message_id={0.welcome_message_id} created_at={0.created_at})".format(
                self
            )
        )

    @property
    def welcome_message_id(self) -> ID:
        """:clasD`: Returns the welcome message's id.

        .. versionadded:: 1.3.5
        """
        return int(self._welcome_message_id)

    @property
    def created_at(self) -> datetime.datetime:
        """:class:`datetime.datetime`: Returns a datetime.datetime object with the WelcomeMessageRule created time.

        .. versionadded:: 1.3.5
        """
        return datetime.datetime.fromtimestamp(int(self._timestamp) / 1000)

    def delete(self):
        """Delete the Welcome Message Rule.

        .. versionadded:: 1.3.5
        """
        self.http_client.request(
            "DELETE",
            "1.1",
            "/direct_messages/welcome_messages/rules/destroy.json",
            params={"id": str(self.id)},
            auth=True,
        )

    def fetch_welcome_message(self) -> Optional[WelcomeMessage]:
        """A method for fetching the welcome message rule's welcome message. An equivalent to :meth:`Client.fetch_welcome_message`.

        Returns
        ---------
        Optional[:class:`WelcomeMessage`]
            This method returns a :class:`WelcomeMessage` object.


        .. versionadded:: 1.5.0
        """
        return self.http_client.fetch_welcome_message(self.welcome_message_id)



"""
pytwot
~~~~~~~

pytwot is a Synchronous python API wrapper for Twitter's API!

:copyright: (c) 2021-present UnrealFar & TheGenocides, 2023-present Sengolda
:license: MIT, see LICENSE for more details.
"""
import logging
from typing import Literal, NamedTuple

from .dataclass import *
from .attachments import *
from .auth import *
from .client import *
from .compliance import *
from .entities import *
from .enums import *
from .environment import *
from .errors import *
from .events import *
from .constants import *
from .http import *
from .list import *
from .message import *
from .objects import *
from .paginations import *
from .parser import *
from .relations import *
from .space import *
from .stream import *
from .tweet import *
from .user import *
from .utils import *

__title__ = "pytwot"
__version__ = "1.5.0a10"
__author__ = "Sengolda"
__license__ = "MIT"
__copyright__ = "Copyright 2021-present UnrealFar & TheGenocides, 2023-present Sengolda"


class VersionInfo(NamedTuple):
    major: int
    minor: int
    micro: int
    releaselevel: Literal["alpha", "beta", "candidate", "final"]
    serial: int


version_info: VersionInfo = VersionInfo(major=1, minor=5, micro=0, releaselevel="alpha", serial=10)

logging.getLogger(__name__).addHandler(logging.NullHandler())

assert version_info.releaselevel in (
    "alpha",
    "beta",
    "candidate",
    "final",
), "Invalid release level given."
